from PIL import Image
import numpy as np
import os
from tqdm import tqdm
from pytomosaic.tileManager import TileManager


VALID_EXTENSIONS = {
	".jpg",
    ".jpeg",
    ".png",
    ".gif",
    ".bmp",
    ".tiff",
    ".tif",
}

def createMosaic(imgPath: str, sourceImages: str, cropSize: int, verbose: bool=False):

	image = Image.open(imgPath)
	width, height = image.size
	cropX, cropY = 0, 0  # top-left corner of the crop
	area = (cropX, cropY, cropX + cropSize, cropY + cropSize)

	if verbose: print("Processing Images...")

	# Skip any files that are not images

	if isinstance(sourceImages, TileManager):
		tileManager = sourceImages
		cropSize = tileManager._cropSize
	else:
		tileManager = TileManager(
			cropSize=cropSize,                 # size of each tile
			sourceImagesDir=sourceImages,    # folder with your source images
			verbose=verbose                 # show loading messages
		)

	if verbose: print("Generating Image...")

	for i in tqdm(range(0, width // cropSize), disable=not verbose):
		for j in range(0, height // cropSize):
			cropX, cropY = i * cropSize, j * cropSize
			area = (cropX, cropY, cropX + cropSize, cropY + cropSize)

			croppedImage = image.crop(area)

			arr = np.array(croppedImage)
			avg = arr.mean(axis=(0,1)).astype(int)

			bestMatch = tileManager.findClosestTile(avg)

			image.paste(bestMatch, (i*cropSize, j*cropSize))


	finalWidth = (width // cropSize) * cropSize
	finalHeight = (height // cropSize) * cropSize

	# Crop and show the result
	return image.crop((0, 0, finalWidth, finalHeight))




from PIL import Image
import numpy as np
import os

VALID_EXTENSIONS = {
    ".jpg", ".jpeg", ".png", ".gif", ".bmp", ".tiff", ".tif"
}

class TileManager:

	def __init__(self, cropSize: int, sourceImagesDir: str, verbose: bool = False):
		self._sourceImagesDir: str = sourceImagesDir
		self.__verbose: bool = verbose
		self._cropSize: int = cropSize
		self.tiles = []
		self.averages = []
		self.__processTiles()

	def __processTiles(self):
		if self.__verbose: print(f"Loading tiles from {self._sourceImagesDir}")

		for mosaicPart in os.listdir(self._sourceImagesDir):
			_, extension = os.path.splitext(mosaicPart)
			if extension not in VALID_EXTENSIONS:
				if self.__verbose: print(f"WARN: File extension '{extension}' is not accepted by PytoMosaic, skipped")
				continue

			mosaicPartPath = os.path.join(self._sourceImagesDir, mosaicPart)
			mosaicPartImg = Image.open(mosaicPartPath).convert('RGB').resize((self._cropSize, self._cropSize))
			arr = np.array(mosaicPartImg)
			mosaicPartProcessed = arr.mean(axis=(0,1)).astype(int)

			self.tiles.append(mosaicPartImg)
			self.averages.append(mosaicPartProcessed)
		
		self.averages = np.array(self.averages)
	
	def findClosestTile(self, target: np.ndarray):
		# Compute distances from target_color to all tile averages
		dists = np.linalg.norm(self.averages - target, axis=1)

		# Find the index of the closest tile
		best_idx = np.argmin(dists)

		# Return the corresponding tile image
		return self.tiles[best_idx]
	
	def getClosestAvg(self, target: np.ndarray):
		
		dists = np.linalg.norm(self.averages - target, axis=1)
		best_idx = np.argmin(dists)
		return self.averages[best_idx]